/*
 *   khexedit - Versatile hex editor
 *   Copyright (C) 1999  Espen Sand, espensa@online.no
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <ctype.h>
#include <iostream.h>

#include <qbuttongroup.h> 
#include <qlabel.h>
#include <qlayout.h>
#include <qmsgbox.h> 
#include <qradiobutton.h>  
#include <qsize.h>
#include <qvalidator.h> 

#include <kapp.h>
#include <kbuttonbox.h>
#include <kfiledialog.h>
#include <kseparator.h>

#include "dialog.h"
#include "dialogmargin.h"

static void entryError( QWidget *parent, const QString &msg );
static const char **formatStrings( void );


CGotoDialog::CGotoDialog( QWidget *parent, const char *name )
  : QDialog( parent, name, false, WStyle_Customize|WStyle_DialogBorder ) 
{
  QVBoxLayout *topLayout = new QVBoxLayout( this, OUTER_SEP, INNER_SEP ); 
  if( topLayout == 0 ) { return; }

  QVBoxLayout *vbox = new QVBoxLayout();
  if( vbox == 0 ) { return; }
  topLayout->addLayout( vbox );

  mComboBox = new QComboBox( true, this );
  if( mComboBox == 0 ) { return; }
  mComboBox->setMaxCount( 10 );
  mComboBox->setInsertionPolicy( QComboBox::AtTop );
  mComboBox->setFixedHeight( mComboBox->sizeHint().height() );
  mComboBox->setMinimumWidth( mComboBox->fontMetrics().width("M") * 20 );  

  QLabel *label = new QLabel( mComboBox, i18n("&Offset"), this );
  if( label == 0 ) { return; }
  label->setFixedSize( label->sizeHint() );

  vbox->addWidget( label, 0, AlignLeft );
  vbox->addWidget( mComboBox, 0, AlignLeft );

  QButtonGroup *group = new QButtonGroup( i18n("Options"), this );
  if( group == 0 ) { return; }
  topLayout->addWidget( group );

  vbox = new QVBoxLayout( group, INNER_SEP, INNER_SEP );
  if( vbox == 0 ) { return; }
  vbox->addSpacing( group->fontMetrics().height() );
  
  mCheckBackward = new QCheckBox( i18n("&Backwards"), group );
  mCheckBackward->setMinimumSize( mCheckBackward->sizeHint() );
  vbox->addWidget( mCheckBackward );
  mCheckFromCursor = new QCheckBox( i18n("&From cursor"), group );
  vbox->addWidget( mCheckFromCursor );
  mCheckFromCursor->setMinimumSize( mCheckFromCursor->sizeHint() );
  vbox->activate();

  //
  // The area above the action buttons (and below the input area)
  // shall take up all available space and be no smaller than the 
  // outer margin size.
  //
  topLayout->addStretch(10);
  topLayout->addSpacing( ACTION_SPACE );

  KButtonBox *buttonBox = new KButtonBox( this, KButtonBox::HORIZONTAL, 0, 
					  INNER_SEP );
  buttonBox->addStretch();
  QPushButton *gotoButton = buttonBox->addButton( i18n("&Apply"), false );
  QPushButton *closeButton = buttonBox->addButton( i18n("&Cancel"), false );
  buttonBox->layout();
  gotoButton->setDefault( true );
  connect( gotoButton, SIGNAL(clicked()),this,SLOT(startGoto()) );
  connect( closeButton, SIGNAL(clicked()),this,SLOT(reject()) );
  topLayout->addWidget( buttonBox );

  topLayout->activate();
  setGeometry( x(), y(), minimumSize().width(), minimumSize().height() );
  defaultFocus();
}

CGotoDialog::~CGotoDialog( void )
{

}




void CGotoDialog::defaultFocus( void )
{
  mComboBox->setFocus();
}


//
// Format of input string:
// 0x<HexNumber>|<DecimalNumber>s<Bit>
// s = :,. or space
//
void CGotoDialog::startGoto( void )
{
  uint offset;
  bool success = stringToOffset( mComboBox->currentText(), offset );
  if( success == false )
  {
    return;
  }

  hide();
  emit gotoOffset( offset, 7, mCheckFromCursor->isChecked(),
		   mCheckBackward->isChecked() == true ? false : true );

  #if 0
  const char *p = mComboBox->currentText();
  if( strlen( p ) == 0 )
  {
    return;
  }

  //
  // Skip any whitespaces in front of string
  //
  for( ; *p != 0 && isspace( *p ) ; p++ );
  
  uint offset, bit;
  int match;
  if( strncmp( p, "0x", 2 ) == 0 || strncmp( p, "0X", 2 ) == 0 )
  {
    match = sscanf( p+2, "%x", &offset );
  }
  else
  {
    match = sscanf( p, "%u", &offset );
  }

  if( match == 0 )
  {
    return;
  }

  bit = 7;

  p = strpbrk( p, ":,. " );
  if( p != 0 )
  {
    match = sscanf( p+1, "%u", &bit );
    if( match == 0 )
    {
      return;
    }
    if( bit > 7 ) { bit = 7; }
  }
  #endif
}




CFindDialog::CFindDialog( QWidget *parent, const char *name )
  : QDialog( parent, name, false, WStyle_Customize|WStyle_DialogBorder ) 
{
  QVBoxLayout *topLayout = new QVBoxLayout( this, OUTER_SEP, INNER_SEP ); 
  if( topLayout == 0 ) { return; }

  QVBoxLayout *vbox = new QVBoxLayout();
  if( vbox == 0 ) { return; }
  topLayout->addLayout( vbox );
  
  mSelector = new QComboBox( false, this );
  if( mSelector == 0 ) { return; }
  mSelector->setFixedHeight( mSelector->sizeHint().height() );
  mSelector->setMinimumWidth( mSelector->fontMetrics().width("M") * 20 );
  mSelector->insertStrList( formatStrings() );
  connect( mSelector, SIGNAL(activated(int)), SLOT(selectorChanged(int)) );
 
  QLabel *label = new QLabel( mSelector, i18n("Fo&rmat"), this );
  if( label == 0 ) { return; }
  label->setFixedSize( label->sizeHint() );

  vbox->addWidget( label, 0, AlignLeft );
  vbox->addWidget( mSelector, 0, AlignLeft );

  mInput = new QLineEdit( this );
  if( mInput == 0 ) { return; }
  mInput->setFixedHeight( mInput->sizeHint().height() );
  mInput->setMinimumWidth( mInput->fontMetrics().width("M") * 20 );
  connect( mInput, SIGNAL(textChanged(const char*)), 
	   SLOT(inputChanged(const char*)) );
  mFindValidator = new CHexValidator( this, CHexValidator::regularText );
  if( mFindValidator == 0 ) { return; }
  mInput->setValidator( mFindValidator );

  label = new QLabel( mInput, i18n("F&ind"), this );
  if( label == 0 ) { return; }
  label->setFixedSize( label->sizeHint() );

  vbox->addWidget( label, 0, AlignLeft );
  vbox->addWidget( mInput, 0, AlignLeft );

  QButtonGroup *group = new QButtonGroup( i18n("Options"), this );
  if( group == 0 ) { return; }
  topLayout->addWidget( group );

  QGridLayout *gbox = new QGridLayout( group, 4, 2, INNER_SEP, INNER_SEP );
  if( gbox == 0 ) { return; }
  gbox->addRowSpacing( 0, group->fontMetrics().height()-INNER_SEP );
  mCheckBackward = new QCheckBox( i18n("&Backwards"), group );
  mCheckBackward->setMinimumSize( mCheckBackward->sizeHint() );
  gbox->addWidget( mCheckBackward, 1, 0 );
  mCheckFromCursor = new QCheckBox( i18n("&From cursor"), group );
  gbox->addWidget( mCheckFromCursor, 2, 0 );
  mCheckIgnoreCase = new QCheckBox( i18n("Ignore cas&e"),group);
  mCheckIgnoreCase->setMinimumSize( mCheckIgnoreCase->sizeHint() );
  gbox->addWidget( mCheckIgnoreCase, 3, 0 );
  mCheckFromCursor->setMinimumSize( mCheckFromCursor->sizeHint() );
  mCheckInSelection = new QCheckBox( i18n("In &selection"), group );
  mCheckInSelection->setMinimumSize( mCheckInSelection->sizeHint() );
  gbox->addWidget( mCheckInSelection, 1, 1 );
  mCheckUseNavigator = new QCheckBox( i18n("&Use navigator"),group);
  mCheckUseNavigator->setMinimumSize( mCheckUseNavigator->sizeHint() );
  gbox->addWidget( mCheckUseNavigator, 2, 1 );


  gbox->activate();

  //
  // The area above the action buttons (and below the input area)
  // shall take up all available space and be no smaller than the 
  // outer margin size.
  //
  topLayout->addStretch(10);
  topLayout->addSpacing( ACTION_SPACE );

  KButtonBox *buttonBox = new KButtonBox( this, KButtonBox::HORIZONTAL, 0, 
					  INNER_SEP );
  buttonBox->addStretch();
  QPushButton *findButton = buttonBox->addButton( i18n("&Apply"), false );
  QPushButton *closeButton = buttonBox->addButton( i18n("&Cancel"), false );
  buttonBox->layout();
  findButton->setDefault( true );
  connect( findButton, SIGNAL(clicked()),this,SLOT(startFind()) );
  connect( closeButton, SIGNAL(clicked()),this,SLOT(reject()) );
  topLayout->addWidget( buttonBox );


  mCheckFromCursor->setChecked( true );
  mCheckUseNavigator->setChecked( true );

  topLayout->activate();
  setGeometry( x(), y(), minimumSize().width(), minimumSize().height() );
  defaultFocus();

}


CFindDialog::~CFindDialog( void )
{
  delete mFindValidator;
}


void CFindDialog::selectorChanged( int index )
{
  mFindValidator->setState( (CHexValidator::EState)index );
  mInput->setText( mFindString[ index ] );
  mCheckIgnoreCase->setEnabled( index == 0 ? true : false );
}

void CFindDialog::inputChanged( const char *text )
{
  mFindString[ mSelector->currentItem() ] = text;
  mFindValidator->convert( mFindData, 
    mFindString[ mSelector->currentItem() ] );
}




void CFindDialog::defaultFocus( void )
{
  mInput->setFocus();
}


const char *CFindDialog::data( void )
{
  return( mFindData.size() == 0 ? 0 : mFindData.data() );
}


uint CFindDialog::size( void )
{
  return( mFindData.size() );
}


void CFindDialog::startFind( void )
{
  if( mFindData.isEmpty() == true )
  {
    entryError( this, QString("") );
    return;
  }

  SSearchControl sc;
  sc.key = mFindData;
  sc.keyType = mSelector->currentItem();
  sc.fromCursor = mCheckFromCursor->isChecked();
  sc.inSelection = mCheckInSelection->isChecked();
  sc.forward = mCheckBackward->isChecked() == true ? false : true;
  if( mCheckBackward->isEnabled() == true )
  {
    sc.ignoreCase = mCheckIgnoreCase->isChecked();
  }
  else
  {
    sc.ignoreCase = false;
  }

  hide();
  emit findData( sc, Find_First, mCheckUseNavigator->isChecked() );
}



void CFindDialog::findAgain( EOperation operation )
{
  if( data() == 0 || size() == 0 )
  {
    return;
  }

  SSearchControl sc;
  sc.key = mFindData;
  sc.fromCursor = true;
  sc.inSelection = mCheckInSelection->isChecked();
  if( operation == find_Next )
  {
    sc.forward = true;
  }
  else if( operation == find_Previous )
  {
    sc.forward = false;
  }
  else
  {
    sc.forward = mCheckBackward->isChecked() == true ? false : true;
  }

  hide();
  emit findData( sc, Find_Next, false );
}




CFindNavigatorDialog::CFindNavigatorDialog( QWidget *parent, const char *name )
  : QDialog( parent, name, false, WStyle_Customize|WStyle_DialogBorder )
{
  QString text;
  QBoxLayout *topLayout = new QVBoxLayout( this, OUTER_SEP, INNER_SEP );
  if( topLayout == 0 ) { return; }

  topLayout->addSpacing( INNER_SEP );

  QHBoxLayout *hbox = new QHBoxLayout();
  if( hbox == 0 ) { return; }
  topLayout->addLayout( hbox );

  text = i18n("Searching for:");
  QLabel *label = new QLabel( text, this );
  label->setFixedHeight( label->sizeHint().height() );
  label->setFixedWidth( label->sizeHint().width() );
  hbox->addWidget( label );
 
  mKey = new QLineEdit( this );
  mKey->setFixedHeight( mKey->sizeHint().height() );
  mKey->setMinimumWidth( fontMetrics().width("M") * 20 );
  mKey->setFocusPolicy( QWidget::NoFocus );
  hbox->addWidget( mKey );

  topLayout->addSpacing( INNER_SEP );

  KSeparator *separator = new KSeparator( this );
  separator->setOrientation( QFrame::HLine );
  topLayout->addWidget( separator );

  topLayout->addStretch(10);
  topLayout->addSpacing( ACTION_SPACE );

  KButtonBox *buttonBox = new KButtonBox( this, KButtonBox::HORIZONTAL, 0, 
					  INNER_SEP );
  buttonBox->addStretch();
  QPushButton *prevButton = buttonBox->addButton( i18n("&Previous"), false );
  QPushButton *nextButton = buttonBox->addButton( i18n("&Next"), false );
  QPushButton *newKeyButton = buttonBox->addButton( i18n("New &key"), false );
  QPushButton *closeButton = buttonBox->addButton( i18n("&Close"), false );
  connect( nextButton, SIGNAL(clicked()), SLOT(next()) );
  connect( prevButton, SIGNAL(clicked()), SLOT(previous()) );
  connect( newKeyButton, SIGNAL(clicked()), SLOT(newKey()) );
  connect( closeButton, SIGNAL(clicked()), SLOT(close()) );
  buttonBox->layout();
  nextButton->setDefault( true );
  topLayout->addWidget( buttonBox );
  nextButton->setFocus();

  topLayout->activate();
  setGeometry( x(), y(), minimumSize().width(), minimumSize().height() );
}


CFindNavigatorDialog::~CFindNavigatorDialog( void )
{
}



void CFindNavigatorDialog::defineData( SSearchControl &sc )
{
  mSearchControl = sc;
  mSearchControl.key.duplicate( sc.key );

  if( mSearchControl.key.isEmpty() == true )
  {
    mKey->setText("");
    return;
  }

  if( mSearchControl.keyType == 0 )
  {
    QString str;
    for( uint i=0; i<mSearchControl.key.size(); i++ )
    {
      str += mSearchControl.key[i];
    }
    mKey->setText( str.data() );


  }
  else if( mSearchControl.keyType == 1 )
  {
    char buf[4];
    QString str("0x ");
    for( uint i=0; i<mSearchControl.key.size(); i++ )
    {
      sprintf( buf, "%02X ", (unsigned char)mSearchControl.key[i] );
      str += buf;
    }
    mKey->setText( str.data() );
  }
  else if( mSearchControl.keyType == 2 )
  {
    char buf[5];
    QString str;
    for( uint i=0; i<mSearchControl.key.size(); i++ )
    {
      sprintf( buf, "%03o ", (unsigned char)mSearchControl.key[i] );
      str += buf;
    }
    mKey->setText( str.data() );
  }
  else
  {
    char buf[10];
    QString str;

    memset( buf, 0, sizeof( buf ) ); buf[8] = ' ';

    for( uint i=0; i<mSearchControl.key.size(); i++ )
    {
      unsigned char data = (unsigned char)mSearchControl.key[i];
      for( int j = 0; j < 8; j++ )
      {
	buf[7-j] = (data&(1<<j)) ? '1' : '0';
      }
      str += buf;
    }
    mKey->setText( str.data() );
  }
}



void CFindNavigatorDialog::next( void ) 
{
  done( repNext );
}


void CFindNavigatorDialog::previous( void ) 
{
  done( repPrevious );
}


void CFindNavigatorDialog::newKey( void ) 
{
  done( repNewKey );
}


void CFindNavigatorDialog::close( void ) 
{
  done( repClose );
}


void CFindNavigatorDialog::done( int resultCode ) 
{
  setResult( resultCode );
  if( resultCode == repClose || resultCode == repNewKey )
  {
    if( resultCode == repNewKey )
    {
      emit makeKey();
    }
    hide();
    return;
  }

  mSearchControl.forward = resultCode == repNext ? true : false;
  emit findData( mSearchControl, Find_Next, true );
}










CReplaceDialog::CReplaceDialog( QWidget *parent, const char *name )
  : QDialog( parent, name, false, WStyle_Customize|WStyle_DialogBorder ) 
{

  QVBoxLayout *topLayout = new QVBoxLayout( this, OUTER_SEP, INNER_SEP ); 
  if( topLayout == 0 ) { return; }

  QVBoxLayout *vbox = new QVBoxLayout();
  if( vbox == 0 ) { return; }
  topLayout->addLayout( vbox );
  

  mFindSelector = new QComboBox( false, this );
  if( mFindSelector == 0 ) { return; }
  mFindSelector->setFixedHeight( mFindSelector->sizeHint().height() );
  mFindSelector->setMinimumWidth( mFindSelector->fontMetrics().width("M")*20 );
  mFindSelector->insertStrList( formatStrings() );
  connect( mFindSelector, SIGNAL(activated(int)), 
	   SLOT(findSelectorChanged(int)) );
  
  QLabel *label = new QLabel( mFindSelector, i18n("Fo&rmat (find)"), this );
  if( label == 0 ) { return; }
  label->setFixedSize( label->sizeHint() );

  vbox->addWidget( label, 0, AlignLeft );
  vbox->addWidget( mFindSelector, 0, AlignLeft );

  mFindInput = new QLineEdit( this );
  if( mFindInput == 0 ) { return; }
  mFindInput->setFixedHeight( mFindInput->sizeHint().height() );
  mFindInput->setMinimumWidth( mFindInput->fontMetrics().width("M")*20 );
  mFindValidator = new CHexValidator( this, CHexValidator::regularText );
  if( mFindValidator == 0 ) { return; }
  mFindInput->setValidator( mFindValidator );
  connect( mFindInput, SIGNAL(textChanged(const char*)), 
	   SLOT(findInputChanged(const char*)) );

  label = new QLabel( mFindInput, i18n("F&ind"), this );
  if( label == 0 ) { return; }
  label->setFixedHeight( label->sizeHint().height() );

  vbox->addWidget( label, 0, AlignLeft );
  vbox->addWidget( mFindInput, 0, AlignLeft );

  mReplaceSelector = new QComboBox( false, this );
  if( mReplaceSelector == 0 ) { return; }
  mReplaceSelector->setFixedHeight( mReplaceSelector->sizeHint().height());
  mReplaceSelector->setMinimumWidth(
    mReplaceSelector->fontMetrics().width("M")*20);
  mReplaceSelector->insertStrList( formatStrings() );
  connect( mReplaceSelector, SIGNAL(activated(int)), 
	   SLOT(replaceSelectorChanged(int)) );

  label = new QLabel( mReplaceSelector, i18n("For&mat (replace)"), this );
  if( label == 0 ) { return; }
  label->setFixedHeight( label->sizeHint().height() );

  vbox->addWidget( label, 0, AlignLeft );
  vbox->addWidget( mReplaceSelector, 0, AlignLeft );

  mReplaceInput = new QLineEdit( this );
  if( mReplaceInput == 0 ) { return; }
  mReplaceInput->setFixedHeight( mReplaceInput->sizeHint().height() );
  mReplaceInput->setMinimumWidth(
    mReplaceInput->fontMetrics().width("M") * 20 );
  mReplaceValidator = new CHexValidator( this, CHexValidator::regularText );
  if( mReplaceValidator == 0 ) { return; }
  mReplaceInput->setValidator( mReplaceValidator );
  connect( mReplaceInput, SIGNAL(textChanged(const char*)), 
	   SLOT(replaceInputChanged(const char*)) );

  label = new QLabel( mReplaceInput, i18n("Rep&lace"), this );
  if( label == 0 ) { return; }
  label->setFixedHeight( label->sizeHint().height() );

  vbox->addWidget( label, 0, AlignLeft );
  vbox->addWidget( mReplaceInput, 0, AlignLeft );

  QButtonGroup *group = new QButtonGroup( i18n("Options"), this );
  if( group == 0 ) { return; }
  topLayout->addWidget( group );

  QGridLayout *gbox = new QGridLayout( group, 4, 2, INNER_SEP, INNER_SEP );
  if( gbox == 0 ) { return; }
  gbox->addRowSpacing( 0, group->fontMetrics().height()-INNER_SEP );
  mCheckBackward = new QCheckBox( i18n("&Backwards"), group );
  mCheckBackward->setMinimumSize( mCheckBackward->sizeHint() );
  gbox->addWidget( mCheckBackward, 1, 0 );
  mCheckFromCursor = new QCheckBox( i18n("&From cursor"), group );
  mCheckFromCursor->setMinimumSize( mCheckFromCursor->sizeHint() );
  gbox->addWidget( mCheckFromCursor, 2, 0 );
  mCheckIgnoreCase = new QCheckBox( i18n("Ignore cas&e"), group );
  mCheckIgnoreCase->setMinimumSize( mCheckIgnoreCase->sizeHint() );
  gbox->addWidget( mCheckIgnoreCase, 3, 0 );
  mCheckInSelection = new QCheckBox( i18n("In &selection"), group );
  mCheckInSelection->setMinimumSize( mCheckInSelection->sizeHint() );
  gbox->addWidget( mCheckInSelection, 1, 1 );
  mCheckPrompt = new QCheckBox( i18n("&Prompt"), group );
  mCheckPrompt->setMinimumSize( mCheckPrompt->sizeHint() );
  gbox->addWidget( mCheckPrompt, 2, 1 );
  gbox->activate();  


  //
  // The area above the action buttons (and below the input area)
  // shall take up all available space and be no smaller than the 
  // outer margin size.
  //
  topLayout->addStretch(10);
  topLayout->addSpacing( ACTION_SPACE );

  KButtonBox *buttonBox = new KButtonBox( this, KButtonBox::HORIZONTAL, 0, 
					  INNER_SEP );
  buttonBox->addStretch();
  QPushButton *findButton = buttonBox->addButton( i18n("&Apply"), false );
  QPushButton *closeButton = buttonBox->addButton( i18n("&Cancel"), false );
  buttonBox->layout();
  findButton->setDefault( true );
  connect( findButton, SIGNAL(clicked()),this,SLOT(startReplace()) );
  connect( closeButton, SIGNAL(clicked()),this,SLOT(reject()) );
  topLayout->addWidget( buttonBox );

  mCheckFromCursor->setChecked( true );
  mCheckPrompt->setChecked( true );

  topLayout->activate();
  setGeometry( x(), y(), minimumSize().width(), minimumSize().height() );
  defaultFocus();
}


CReplaceDialog::~CReplaceDialog( void )
{
  delete mFindValidator;
  delete mReplaceValidator;
}




void CReplaceDialog::findSelectorChanged( int index )
{
  mFindValidator->setState( (CHexValidator::EState)index );
  mFindInput->setText( mFindString[ index ] );
  mCheckIgnoreCase->setEnabled( index == 0 ? true : false );
}

void CReplaceDialog::findInputChanged( const char *text )
{
  mFindString[ mFindSelector->currentItem() ] = text;
  mFindValidator->convert( mFindData, 
    mFindString[ mFindSelector->currentItem() ] );
}

void CReplaceDialog::replaceSelectorChanged( int index )
{
  mReplaceValidator->setState( (CHexValidator::EState)index );
  mReplaceInput->setText( mReplaceString[ index ] );
}

void CReplaceDialog::replaceInputChanged( const char *text )
{
  mReplaceString[ mReplaceSelector->currentItem() ] = text;
  mReplaceValidator->convert( mReplaceData, 
    mReplaceString[ mReplaceSelector->currentItem() ] );
}

void CReplaceDialog::defaultFocus( void )
{
  mFindInput->setFocus();
}


const char *CReplaceDialog::findData( void )
{
  return( mFindData.size() == 0 ? 0 : mFindData.data() );
}

const char *CReplaceDialog::replaceData( void )
{
  return( mReplaceData.size() == 0 ? 0 : mReplaceData.data() );
}

uint CReplaceDialog::findSize( void )
{
  return( mFindData.size() );
}

uint CReplaceDialog::replaceSize( void )
{
  return( mReplaceData.size() );
}



void CReplaceDialog::startReplace( void )
{
  if( mFindData.isEmpty() == true )
  {
    entryError( this, QString("") );
    return;
  }

  if( mFindData == mReplaceData )
  {
    entryError( this, i18n("Source and target values can not be equal.") );
    return;
  }

  hide();

  SSearchControl sc;
  sc.key = mFindData;
  sc.val = mReplaceData;
  sc.fromCursor = mCheckFromCursor->isChecked();
  sc.inSelection = mCheckInSelection->isChecked();
  sc.forward = mCheckBackward->isChecked() == true ? false : true;

  if( mCheckBackward->isEnabled() == true )
  {
    sc.ignoreCase = mCheckIgnoreCase->isChecked();
  }
  else
  {
    sc.ignoreCase = false;
  }

  if( mCheckPrompt->isChecked() == true )
  {
    emit replaceData( sc, Replace_First );
  }
  else
  {
    emit replaceData( sc, Replace_AllInit );
  }
}





CReplacePromptDialog::CReplacePromptDialog( QWidget *parent, const char *name )
  : QDialog( parent, name, false, WStyle_Customize|WStyle_DialogBorder )
{
  QString text;
  QBoxLayout *topLayout = new QVBoxLayout( this, OUTER_SEP, INNER_SEP );
  if( topLayout == 0 ) { return; }

  topLayout->addSpacing( INNER_SEP );

  text = i18n("Replace marked data at cursor position?");
  QLabel* label = new QLabel( text, this );
  label->setFixedHeight( label->sizeHint().height() );
  label->setMinimumWidth( label->sizeHint().width() );
  topLayout->addWidget( label );

  topLayout->addSpacing( INNER_SEP );
  
  KSeparator *separator = new KSeparator( this );
  separator->setOrientation( QFrame::HLine );
  topLayout->addWidget( separator );

  topLayout->addStretch(10);
  topLayout->addSpacing( ACTION_SPACE );

  KButtonBox *buttonBox = new KButtonBox( this, KButtonBox::HORIZONTAL, 0, 
					  INNER_SEP );
  buttonBox->addStretch();
  QPushButton *yesButton = buttonBox->addButton( i18n("&Yes"), false );
  QPushButton *noButton = buttonBox->addButton( i18n("&No"), false );
  QPushButton *allButton = buttonBox->addButton( i18n("&All"), false );
  QPushButton *closeButton = buttonBox->addButton( i18n("&Close"), false );
  connect( yesButton,SIGNAL(clicked()),this,SLOT(yes()));
  connect( noButton,SIGNAL(clicked()),this,SLOT(no()));
  connect( allButton,SIGNAL(clicked()),this,SLOT(all()));
  connect( closeButton, SIGNAL(clicked()),this,SLOT(close()) );
  buttonBox->layout();
  yesButton->setDefault( true );
  topLayout->addWidget( buttonBox );
  yesButton->setFocus();

  topLayout->activate();
  setGeometry( x(), y(), minimumSize().width(), minimumSize().height() );
}


CReplacePromptDialog::~CReplacePromptDialog( void )
{
}

void CReplacePromptDialog::defineData( SSearchControl &sc )
{
  mSearchControl = sc;
  mSearchControl.key.duplicate( sc.key );
  mSearchControl.val.duplicate( sc.val );
}


void CReplacePromptDialog::no( void ) 
{
  done( repNo );
}


void CReplacePromptDialog::yes( void ) 
{
  done( repYes );
}


void CReplacePromptDialog::all( void ) 
{
  done( repAll );
}


void CReplacePromptDialog::close( void ) 
{
  done( repClose );
}


void CReplacePromptDialog::done( int returnCode ) 
{
  if( returnCode == repClose )
  {
    hide();
  }
  else if( returnCode == repYes )
  {
    emit replaceData( mSearchControl, Replace_Next );
  }
  else if( returnCode == repNo )
  {
    emit replaceData( mSearchControl, Replace_Ignore );
  }
  else
  {
    emit replaceData( mSearchControl, Replace_All );
  }
}



CInsertDialog::CInsertDialog( QWidget *parent, const char *name )
  : QDialog( parent, name, false, WStyle_Customize|WStyle_DialogBorder ) 
{

  QVBoxLayout *topLayout = new QVBoxLayout( this, OUTER_SEP, INNER_SEP ); 
  if( topLayout == 0 ) { return; }

  QVBoxLayout *vbox = new QVBoxLayout();
  if( vbox == 0 ) { return; }
  topLayout->addLayout( vbox );

  mSizeBox = new QSpinBox( this );
  if( mSizeBox == 0 ) { return; }
  mSizeBox->setFixedHeight( mSizeBox->sizeHint().height() );
  mSizeBox->setMinimumWidth( mSizeBox->fontMetrics().width("M")*20 );
  mSizeBox->setRange( 1, 1000000 );
  
  QLabel *label = new QLabel( mSizeBox, i18n("&Size"), this );
  if( label == 0 ) { return; }
  label->setFixedHeight( label->sizeHint().height() );

  vbox->addWidget( label, 0, AlignLeft );
  vbox->addWidget( mSizeBox, 0, AlignLeft );

  mPatternSelector = new QComboBox( false, this );
  if( mPatternSelector == 0 ) { return; }
  mPatternSelector->setFixedHeight( mPatternSelector->sizeHint().height());
  mPatternSelector->setMinimumWidth(
    mPatternSelector->fontMetrics().width("M")*20);
  mPatternSelector->insertStrList( formatStrings() );
  connect( mPatternSelector, SIGNAL(activated(int)),
	   SLOT(patternSelectorChanged(int)) );

  label = new QLabel( mPatternSelector, i18n("Fo&rmat (pattern)"), this );
  if( label == 0 ) { return; }
  label->setFixedHeight( label->sizeHint().height() );
 
  vbox->addWidget( label, 0, AlignLeft );
  vbox->addWidget( mPatternSelector, 0, AlignLeft );

  mPatternInput = new QLineEdit( this );
  if( mPatternInput == 0 ) { return; }
  mPatternInput->setFixedHeight( mPatternInput->sizeHint().height() );
  mPatternInput->setMinimumWidth( mPatternInput->fontMetrics().width("M")*20 );
  mPatternValidator = new CHexValidator( this, CHexValidator::regularText );
  if( mPatternValidator == 0 ) { return; }
  mPatternInput->setValidator( mPatternValidator );
  connect( mPatternInput, SIGNAL(textChanged(const char*)), 
	   SLOT(patternInputChanged(const char*)) );

  label = new QLabel( mPatternInput, i18n("&Pattern"), this );
  if( label == 0 ) { return; }
  label->setFixedHeight( label->sizeHint().height() );

  vbox->addWidget( label, 0, AlignLeft );
  vbox->addWidget( mPatternInput, 0, AlignLeft );

  mOffsetInput = new QLineEdit( this );
  mOffsetInput->setFixedHeight( mOffsetInput->sizeHint().height() );
  mOffsetInput->setMinimumWidth( mOffsetInput->fontMetrics().width("M")*20 );

  mOffsetLabel = new QLabel( mOffsetInput, i18n("&Offset"), this );
  if( mOffsetLabel == 0 ) { return; }
  mOffsetLabel->setFixedHeight( mOffsetLabel->sizeHint().height() );

  vbox->addWidget( mOffsetLabel, 0, AlignLeft );
  vbox->addWidget( mOffsetInput, 0, AlignLeft );

  QButtonGroup *group = new QButtonGroup( i18n("Options"), this );
  if( group == 0 ) { return; }
  topLayout->addWidget( group );

  vbox = new QVBoxLayout( group, INNER_SEP, INNER_SEP );
  if( vbox == 0 ) { return; }
  vbox->addSpacing( group->fontMetrics().height() );
  
  mCheckPattern = new QCheckBox( i18n("R&epeat pattern"), group );
  mCheckPattern->setMinimumSize( mCheckPattern->sizeHint() );
  vbox->addWidget( mCheckPattern );
  mCheckOnCursor = new QCheckBox( i18n("&Insert on cursor position"), group );
  vbox->addWidget( mCheckOnCursor );
  mCheckOnCursor->setMinimumSize( mCheckOnCursor->sizeHint() );
  connect( mCheckOnCursor, SIGNAL(clicked()),this,SLOT(cursorCheck()) );
  vbox->activate();


  //
  // The area above the action buttons (and below the input area)
  // shall take up all available space and be no smaller than the 
  // outer margin size.
  //
  topLayout->addStretch(10);
  topLayout->addSpacing( ACTION_SPACE );

  KButtonBox *buttonBox = new KButtonBox( this, KButtonBox::HORIZONTAL, 0, 
					  INNER_SEP );
  buttonBox->addStretch();
  QPushButton *findButton = buttonBox->addButton( i18n("&Apply"), false );
  QPushButton *closeButton = buttonBox->addButton( i18n("&Cancel"), false );
  buttonBox->layout();
  findButton->setDefault( true );
  connect( findButton, SIGNAL(clicked()),this,SLOT(startInsert()) );
  connect( closeButton, SIGNAL(clicked()),this,SLOT(reject()) );
  topLayout->addWidget( buttonBox );

  mCheckOnCursor->setChecked( true );
  cursorCheck();

  topLayout->activate();
  setGeometry( x(), y(), minimumSize().width(), minimumSize().height() );
  defaultFocus();
}

CInsertDialog::~CInsertDialog( void )
{
  delete mPatternValidator;
}

void CInsertDialog::defaultFocus( void )
{
  mSizeBox->setFocus();
}




void CInsertDialog::patternSelectorChanged( int index )
{
  mPatternValidator->setState( (CHexValidator::EState)index );
  mPatternInput->setText( mPatternString[ index ] );
}

void CInsertDialog::patternInputChanged( const char *text )
{
  mPatternString[ mPatternSelector->currentItem() ] = text;
  mPatternValidator->convert( mPatternData, 
    mPatternString[ mPatternSelector->currentItem() ] );
}

const char *CInsertDialog::pattern( void )
{
  return( mPatternData.size() == 0 ? 0 : mPatternData.data() );
}

uint CInsertDialog::patternSize( void )
{
  return( mPatternData.size() );
}


void CInsertDialog::startInsert( void )
{
  if( mPatternData.isEmpty() == true )
  {
    entryError( this, QString("") );
    return;
  }

  SInsertData id;
  id.size = mSizeBox->value();
  id.repeatPattern = mCheckPattern->isChecked();
  id.onCursor = mCheckOnCursor->isChecked();
  id.pattern = pattern();
  id.patternSize = patternSize();

  if( id.onCursor == false )
  {
    bool success = stringToOffset( mOffsetInput->text(), id.offset );
    if( success == false )
    {
      entryError( this, QString("") );
      return;
    }
  }

  hide();
  execute( id );
}


void CInsertDialog::cursorCheck( void )
{
  bool state = mCheckOnCursor->isChecked() == true ? false : true;
  mOffsetLabel->setEnabled( state );
  mOffsetInput->setEnabled( state );
}




CExportDialog::CExportDialog( QWidget *parent, const char *name )
  : QDialog( parent, name, false, WStyle_Customize|WStyle_DialogBorder ) 
{

  QVBoxLayout *topLayout = new QVBoxLayout( this, OUTER_SEP, INNER_SEP ); 
  if( topLayout == 0 ) { return; }

  QVBoxLayout *vbox = new QVBoxLayout();
  if( vbox == 0 ) { return; }
  topLayout->addLayout( vbox );

 
  mDestinationLabel = new QLabel( i18n("Destination"),this);
  if( mDestinationLabel == 0 ) { return; }
  mDestinationLabel->setFixedHeight( mDestinationLabel->sizeHint().height() );
  vbox->addWidget( mDestinationLabel, 0, AlignLeft );

  QHBoxLayout *hbox = new QHBoxLayout();
  if( hbox == 0 ) { return; }
  topLayout->addLayout( hbox );

  mDestinationInput = new QLineEdit( this );
  mDestinationInput->setMinimumWidth( 
    mDestinationInput->fontMetrics().width("M")*20 );
  //Fixed height, see below
  hbox->addWidget( mDestinationInput, 0, AlignTop );

  const char *text = i18n("&Browse");
  QPushButton *button = new QPushButton( text, this, "browse" );
  button->setFixedSize( button->sizeHint() );
  mDestinationInput->setFixedHeight( button->sizeHint().height() );
  hbox->addWidget( button, 0, AlignTop );
  connect( button, SIGNAL(clicked()), SLOT(browserClicked()) );

  QButtonGroup *group = new QButtonGroup( this );
  if( group == 0 ) { return; }
  group->setFrameStyle( QFrame::NoFrame );
  topLayout->addWidget( group );

  vbox = new QVBoxLayout( group, INNER_SEP, INNER_SEP );
  if( vbox == 0 ) { return; }
 
  mRadioEverything = new QRadioButton( i18n("&Everything"), group );
  mRadioEverything ->setMinimumSize( mRadioEverything->sizeHint() );
  vbox->addWidget( mRadioEverything );
  mRadioSelectedArea = new QRadioButton( i18n("&Selected Area (lines)"),group);
  mRadioSelectedArea->setMinimumSize( mRadioSelectedArea->sizeHint() );
  vbox->addWidget( mRadioSelectedArea );
  mRadioSpecify = new QRadioButton( i18n("Spec&ify (offset within line)"), 
				    group );
  mRadioSpecify->setMinimumSize( mRadioSpecify->sizeHint() );
  vbox->addWidget( mRadioSpecify );
  vbox->activate();
  connect( group, SIGNAL(clicked(int)), SLOT(radioClicked(int)) );


  group = new QButtonGroup( this );
  if( group == 0 ) { return; }
  group->setFrameStyle( QFrame::NoFrame );
  topLayout->addWidget( group );

  QGridLayout *gbox = new QGridLayout( group, 2, 2, 0, INNER_SEP );
  if( gbox == 0 ) { return; }

  mFromInput = new QLineEdit( group );
  mFromInput->setFixedHeight( mFromInput->sizeHint().height() );
  gbox->addWidget( mFromInput, 1, 0 );
  mToInput = new QLineEdit( group );
  mToInput->setFixedHeight( mToInput->sizeHint().height() );
  gbox->addWidget( mToInput, 1, 1 );
  mFromLabel = new QLabel( mFromInput, i18n("&From"), group );
  mFromLabel->setFixedHeight( mFromLabel->sizeHint().height() );
  gbox->addWidget( mFromLabel, 0, 0 );
  mToLabel = new QLabel( mToInput, i18n("&To"), group );
  mToLabel->setFixedHeight( mToLabel->sizeHint().height() );
  gbox->addWidget( mToLabel, 0, 1 );
  gbox->activate();
 

  group = new QButtonGroup( i18n("Options"), this );
  if( group == 0 ) { return; }
  topLayout->addWidget( group );

  vbox = new QVBoxLayout( group, INNER_SEP, INNER_SEP );
  if( vbox == 0 ) { return; }
  vbox->addSpacing( group->fontMetrics().height() );
  
  mCheckPS = new QCheckBox( i18n("Output as &postscript"), group );
  mCheckPS->setMinimumSize( mCheckPS->sizeHint() );
  vbox->addWidget( mCheckPS );
  mCheckStdout = new QCheckBox( i18n("&Dump to stdout"), group );
  vbox->addWidget( mCheckStdout );
  mCheckStdout->setMinimumSize( mCheckStdout->sizeHint() );
  vbox->activate();


  //
  // The area above the action buttons (and below the input area)
  // shall take up all available space and be no smaller than the 
  // outer margin size.
  //
  topLayout->addStretch(10);
  topLayout->addSpacing( ACTION_SPACE );

  KButtonBox *buttonBox = new KButtonBox( this, KButtonBox::HORIZONTAL, 0, 
					  INNER_SEP );
  buttonBox->addStretch();
  QPushButton *findButton = buttonBox->addButton( i18n("&Apply"), false );
  QPushButton *closeButton = buttonBox->addButton( i18n("&Cancel"), false );
  buttonBox->layout();
  findButton->setDefault( true );
  connect( findButton, SIGNAL(clicked()),this,SLOT(startExport()) );
  connect( closeButton, SIGNAL(clicked()),this,SLOT(reject()) );
  topLayout->addWidget( buttonBox );


  mRadioEverything->setChecked( true );
  radioClicked( 0 );
  mDestinationInput->setText( "hexdump.txt" );
  mCheckPS->setEnabled( false );

  topLayout->activate();
  setGeometry( x(), y(), minimumSize().width(), minimumSize().height() );
  defaultFocus();
}


CExportDialog::~CExportDialog( void )
{
}


void CExportDialog::defaultFocus( void )
{
  mDestinationInput->setFocus();
}


void CExportDialog::radioClicked( int id )
{
  bool state = id == 2 ? true : false; 
  mFromLabel->setEnabled( state );
  mToLabel->setEnabled( state );
  mFromInput->setEnabled( state );
  mToInput->setEnabled( state );
}

void CExportDialog::checkStdout( void )
{
  bool state = mCheckStdout->isChecked() ? false : true;
  mDestinationLabel->setEnabled( state );
  mDestinationInput->setEnabled( state );
}



void CExportDialog::startExport( void )
{
  SExportData ed;

  ed.url = mDestinationInput->text();

  if( mRadioEverything->isChecked() )
  {
    ed.mode = SExportData::Everything;
  }
  else if( mRadioSelectedArea->isChecked() )
  {
    ed.mode = SExportData::SelectedArea;
  }
  else if( mRadioSpecify->isChecked() )
  {
    ed.mode = SExportData::Specified;
    bool ok1 = stringToOffset( mFromInput->text(), ed.start );
    bool ok2 = stringToOffset( mToInput->text(), ed.stop );
    if( ok1 == false || ok2 == false )
    {
      QString msg = i18n("Illegal format or missing offset specification.");
      entryError( this, msg );
      return;
    }
  }
  else
  {
    return;
  }

  ed.asPostScript = mCheckPS->isChecked();
  ed.toStdout = mCheckStdout->isChecked();

  ed.success = false;
  execute( ed );
  if( ed.success == true )
  {
    hide();
  }
}


void CExportDialog::browserClicked( void )
{
  QString url = KFileDialog::getSaveFileName( 0, "*", topLevelWidget() );
  if( url.isEmpty() == true )
  {
    return;
  }

  mDestinationInput->setText( url.data() );
}









void centerDialog( QWidget *widget, QWidget *centerParent )
{ 
  if( centerParent == 0 || widget == 0 )
  {
    return;
  }

  QPoint point = centerParent->mapToGlobal( QPoint(0,0) );
  QRect pos    = centerParent->geometry();
 
  widget->setGeometry( point.x() + pos.width()/2  - widget->width()/2,
		       point.y() + pos.height()/2 - widget->height()/2, 
		       widget->width(), widget->height() );
}


void centerDialogBottom( QWidget *widget, QWidget *centerParent )
{ 
  if( centerParent == 0 || widget == 0 )
  {
    return;
  }

  QPoint point = centerParent->mapToGlobal( QPoint(0,0) );
  QRect pos    = centerParent->geometry();

  widget->setGeometry( point.x() + pos.width()/2  - widget->width()/2,
		       point.y() + pos.height() - widget->height(), 
		       widget->width(), widget->height() );
}


void comboMatchText( QComboBox *combo, const char *text )
{
  if( text != 0 )
  {
    for( int i=0; i < combo->count(); i++ )
    {
      if( strcmp( combo->text(i), text ) == 0 ) 
      { 
	combo->setCurrentItem(i); 
	return;
      }
    }
  }

  combo->setCurrentItem(0); 
}





void entryError( QWidget *parent, const QString &msg )
{
  QString message;
  message += i18n("Your request can not be processed.");
  message += "\n";
  if( msg.isNull() == true || msg.isEmpty() == true )
  {
    message += i18n("Examine argument(s) and try again.");
  }
  else
  {
    message += msg;
  }
    
  QMessageBox::warning(
    parent,
    i18n("Invalid argument(s)"),
    message.data(),
    i18n("&OK"),
    "",
    "",
    0,0 );
}


bool stringToOffset( const char *text, uint &offset )
{
  if( text == 0 || strlen(text) == 0 )
  {
    return( false );
  }

  const char *p = text;

  //
  // Skip any whitespaces in front of string
  //
  for( ; *p != 0 && isspace( *p ) ; p++ );
  
  int match = 0;
  bool space = false; 
  if( strncmp( p, "0x", 2 ) == 0 || strncmp( p, "0X", 2 ) == 0 )
  {
    for( const char *q = p+2; *q != 0; q++ )
    {
      if( isxdigit( *q ) == 0 || space == true )
      {
	if( isspace( *q ) == 0 )
	{
	  return( false );
	}
	space = true;
      }
    }
    match = sscanf( p+2, "%x", &offset );
  }
  else
  {
    for( const char *q = p; *q != 0; q++ )
    {
      if( isdigit( *q ) == 0 || space == true )
      {
	if( isspace( *q ) == 0 )
	{
	  return( false );
	}
	space = true;
      }
    }
    match = sscanf( p, "%u", &offset );
  }

  if( match == 0 )
  {
    return( false );
  }
  
  return( true );
}


static const char **formatStrings( void )
{
  //FIXME: change to QString when translate returns QStrings.
  static const char *items[] = 
  { 
    i18n( "Regular text" ),
    i18n( "Hexadecimal" ),
    i18n( "Octal" ),
    i18n( "Binary" ),
    0 
  };
  return( items );
}



